package LCM::Gui::TaskExecutionPlan;

use Wx qw(wxEXPAND wxVERTICAL wxSUNKEN_BORDER wxWHITE
         wxALIGN_CENTER_VERTICAL wxVSCROLL wxALL);

use SAPDB::Install::Resources;
use SDB::Install::Gui::Design;
use strict;

our @ISA = qw(Wx::ScrolledWindow);

sub new{
    my ($self) = shift->SUPER::new(@_,[-1,-1],[-1,-1],wxSUNKEN_BORDER);
    $self->SetBackgroundColour (wxWHITE);
    $self->{bmps} = {};
    my $imgs = new SAPDB::Install::Resources()->getImages ('icon/ready.png','icon/process.png','icon/error.png');
    if (defined $imgs){
        $self->{bmps}->{'RUNNING'} = Wx::Bitmap->new ($imgs->{'icon/process.png'});
        $self->{bmps}->{'FINISHED_SUCCESS'} = Wx::Bitmap->new ($imgs->{'icon/ready.png'});
        $self->{bmps}->{'FINISHED_ERROR'} = Wx::Bitmap->new ($imgs->{'icon/error.png'});
    }
    else{
        die ("resource error: cannot get images\n");
    }   
    return $self;
}

sub init{
    my ($self,$plannedTasks,$configuration) = @_;
    return if(@{$plannedTasks} == 0 );
    $self->{plannedTasks} = $plannedTasks;
    delete $self->{'current_step'};
    delete $self->{'done'};
    $self->{'config'} = $configuration;
    my $sizer = Wx::FlexGridSizer->new (scalar @$plannedTasks, 3, 10, 5);
    $sizer->AddGrowableCol (2);
    
    my $app = Wx::wxTheApp();
    my $instconfig = $app->getInstconfig();
    my $taskInfos = {};
    my $first = 1;
    my @persistedSteps = ();
    if ($instconfig->isUpdate() || $instconfig->isPendingInstallation()) {
        @persistedSteps = split(',', $instconfig->getPersistedSteps() // '');
    }
    foreach my $task (@$plannedTasks){
        my $taskInfo = {};
        $taskInfo->{'bmp'} = Wx::StaticBitmap->new($self,-1,$self->{'bmps'}->{'RUNNING'});
        if ($first){
            $first = 0;
        }
        else{
            $taskInfo->{'bmp'}->Show (0);
        }
        $sizer->Add ($taskInfo->{'bmp'},0,wxALIGN_CENTER_VERTICAL);
        $taskInfo->{'label'} = Wx::StaticText->new($self,-1,$task->getExecutionName());
        $taskInfo->{'label'}->SetFont (SDBFONT_NormalText) if (!Wx::wxGTK1());
        $sizer->Add ($taskInfo->{'label'},0,wxALIGN_CENTER_VERTICAL);
        if($configuration->{'with_progress'}){
            $taskInfo->{'gauge'} = Wx::Gauge->new ($self, -1, 100, [-1,-1], [150,15]);
            $sizer->Add ($taskInfo->{'gauge'},1,wxEXPAND | wxALIGN_CENTER_VERTICAL);
        }
        else{
            $sizer->AddStretchSpacer (1);
        }
        $taskInfos->{$task->getId()} = $taskInfo;

        my $stepName = $task->getStepName();
        if (@persistedSteps && grep {$_ eq $stepName} @persistedSteps) {
            $taskInfo->{bmp}->Show (1);
            $taskInfo->{bmp}->SetBitmap($self->{bmps}->{"FINISHED_SUCCESS"});
            $taskInfo->{gauge}->SetValue (100);
        }
    }
    $self->SetScrollRate(0,5);
    $self->{taskInfos} = $taskInfos;
    my $top_sizer = Wx::BoxSizer->new (wxVERTICAL);
    
    $top_sizer->Add($sizer,1,wxEXPAND | wxALL, 10);
    $self->SetSizer ($top_sizer);
    $self->SetAutoLayout (1);
    $self->Layout ();
    
    return 1;
}

sub IsCurrentStepVisible{
    my ($self) = @_;
    
    my ($scrollx_unit,$scrolly_unit) = $self->GetScrollPixelsPerUnit ();
    my $cur_pos = $self->{taskInfos}->{$self->{'current_step'}}->{bmp}->GetPosition();
    my ($cur_pos_x,$cur_pos_y) = $self->CalcUnscrolledPosition($cur_pos->x,$cur_pos->y);
    my ($start_x,$start_y) = $self->GetViewStart ();
    
    $start_y *=  $scrolly_unit;
    
    if ($cur_pos->y < $cur_pos_y){
        return 0;
    }   
    
    
    #my ($width,$height) = $self->GetSizeWH ();
    my ($width,$height) = $self->GetClientSize ();
    if (!defined $height){
        $height = 0;
    }
    if ($cur_pos_y  + 30 > $start_y + $height){
        return 0;
    }   
    return 1;
}

sub ScrollToCurrentStep{
    my ($self) = @_;
    my ($scrollx_unit,$scrolly_unit) = $self->GetScrollPixelsPerUnit ();
    my $cur_pos = $self->{taskInfos}->{$self->{'current_step'}}->{label}->GetPosition();
    
    my ($cur_pos_x,$cur_pos_y) = $self->CalcUnscrolledPosition($cur_pos->x,$cur_pos->y);
    my ($start_x,$start_y) = $self->GetViewStart ();
    
    $start_y *=  $scrolly_unit;
    
    if ($cur_pos_y < $start_y){
        $self->Scroll (0,$cur_pos->y / $scrolly_unit);
        return 0;
    }   
    
    my ($width,$height) = $self->GetSizeWH ();
    
    if ($cur_pos_y + 30 > $start_y + $height){
        my $val = int (($cur_pos_y + 30  - $height) / $scrolly_unit);
        $self->Scroll (-1,$val);
        return 0;
    }   
    return 1;
}

sub update{
	my ($self, $task) = @_;
	if($task->getStatus()->isInInitialState()){
    	return;
    }
	$self->{current_step} = $task->getId();
	my $taskInfo = $self->{taskInfos}->{$self->{current_step}};
	$taskInfo->{bmp}->Show (1);
    if($task->getStatus()->isInFinishedState()){
        $taskInfo->{bmp}->SetBitmap($self->{bmps}->{"FINISHED_SUCCESS"});
    } elsif ($task->getStatus()->isInErrorState() || $task->getStatus()->isInAbortedState()){
        $taskInfo->{bmp}->SetBitmap($self->{bmps}->{"FINISHED_ERROR"});
    }
    
    $taskInfo->{gauge}->SetValue ($task->getProgress());
    
	unless ($self->IsCurrentStepVisible ()){
        $self->ScrollToCurrentStep ();
    }
    Wx::Yield ();
    return 1;
}

sub Destroy{
    my ($self) = @_;
    my $sizer = $self->GetSizer();
    if (defined $sizer){
        $sizer->Clear (1);
    }
    $self->SUPER::Destroy;
}


sub DESTROY{
    my ($self) = @_;
    print ">>>>> destroying $self\n"
        if defined $SAPDB::Install::DEBUG::DESTRUCT &&
            $SAPDB::Install::DEBUG::DESTRUCT;
}

1;
