package SDB::Install::App::Console::HdbMigrateSecureStore;

use strict;
use warnings;
use parent 'SDB::Install::App::Console';

use Getopt::Long;
use SDB::Install::SAPInstance::TrexInstance;
use SDB::Install::SAPInstance::NameserverUtility;
use SDB::Install::Globals qw($gProductNameLSS);
use SDB::Install::Configuration::AnyModifyConfig;

my @secureStoreTypes = ('LSS', 'SSFS');

sub new {
    my $self = shift->SUPER::new();
    $self->{instconfig} = new SDB::Install::Configuration::AnyModifyConfig();
    return $self;
}

sub init {
    my ($self) = @_;

    return undef if (!$self->defineLog(
                        $self->getProgramName(),
                        $self->getProgramName(),
                        undef,
                        $self->{options}->{sid}
    ));

    my $instance  = SDB::Install::SAPInstance::TrexInstance->new(
                        $self->{options}->{path},
                        $self->{options}->{sid}
                    );
    $instance->setMsgLstContext($self->getMsgLstContext());

    my $nsutility = SDB::Install::SAPInstance::NameserverUtility->new($instance);
    $nsutility->setMsgLstContext($self->getMsgLstContext());

    my $rc = $nsutility->migrateSecureStore($self->{options}->{target});

    return $rc;
}

sub checkParamSid {
    my ($self) = @_;
    if ($self->{instconfig}->checkExistingSID($self->{options}->{sid})) {
        return 1;
    }
    $self->getMsgLst()->addError("SID ".$self->{options}->{sid}." does not exist");
    return 0;
}

sub checkParamPath {
    my ($self) = @_;
    if ($self->{instconfig}->checkTarget($self->{options}->{path})) {
        return 1;
    }
    $self->getMsgLst()->addError("Path ".$self->{options}->{path}." does not exist");
    return 0;
}

sub checkParamTarget {
    my ($self) = @_;
    foreach my $type (@secureStoreTypes) {
        return 1 if ($type eq $self->{options}->{target});
    }
    $self->getMsgLst()->addError("Invalid target: \"".$self->{options}->{target}."\"");
    return 0;
}

sub getAction {
    return "Migrate";
}

sub getActionDone {
    return "migrated";
}

sub getActionProgressive {
    return "Migrating";
}

sub getProgramName {
    return 'hdbmigratesecurestore';
}

sub InitCmdLineArgs {
    my ($self, $args) = @_;

    $self->{options} = {
        'sid'    => undef,
        'path'   => undef,
        'target' => undef
    };

    # add/reuse param checks
    GetOptions(
        'sid=s'    => \$self->{options}->{sid},
        'path=s'   => \$self->{options}->{path},
        'target=s' => \$self->{options}->{target}
    );

    return 1 if (
        $self->checkParamSid() &&
        $self->checkParamPath() &&
        $self->checkParamTarget()
    );

    return undef;
}

sub main {
    my $app = new __PACKAGE__;
    $app->mainFunction(\@_);
}

sub shouldWarnIfCalledStandalone {
    return 1;
}

1;
