package SDB::Install::Configuration::ValueChangeListeners::ConfigureFastRestartOptionListener;

use strict;
use warnings;
use SDB::Install::System qw(which);

sub new {
    my ($class) = @_;
    return bless({}, $class);
}

sub onValueChange {
    my ($self, $value, $configuration) = @_;

    $configuration->setSkip('TmpfsMountpointsPath', !$value);
    $configuration->setSkip('PmemGlobalAllocationLimit', !$value);
    return 1 if(!$value);

    return 0 if !$self->handleIncompatibleParameters($configuration);
    return 1 if defined $configuration->getBatchValue('PmemGlobalAllocationLimit');

    # Default value should be 50% of the available physical memory
    my $availableMemory = $self->getAvailableMemory($configuration);
    if(defined $availableMemory) {
        $configuration->setDefault('PmemGlobalAllocationLimit', int($availableMemory / 2));
    }
    return 1;
}

sub handleIncompatibleParameters {
    my ($self, $configuration) = @_;
    foreach my $paramId ('BasePathPMem', 'PMemMountPoints'){
        if($configuration->getBatchValue($paramId)){
            my $option = $configuration->getOpt($paramId);
            $configuration->getErrMsgLst()->addError("Fast restart option cannot be configured when $option is used.");
            return 0;
        }
    }
    return 1;
}

sub getAvailableMemory {
    my ($self, $configuration) = @_;
    $configuration->getMsgLst()->addMessage("Checking available memory...");
    my $freePath = which('free');
    my $executor = LCM::ProcessExecutor->new($freePath, ['-m']);
    $executor->setMsgLstContext($configuration->getMsgLstContext());
    my $exitCode = $executor->executeProgram();
    if(!defined $exitCode || $exitCode){
        $configuration->getErrMsgLst()->addError("Available memory check failed. Cannot determine 'persistent_memory_global_allocation_limit' paramater default value");
    }

    my $availableMemory;
    foreach my $line (@{$executor->getOutputLines()}) {
        ($availableMemory) = $line =~ qr/^Mem:\s+(\d+)/;
        last if($availableMemory);
    }

    return $availableMemory;
}

1;
