package SDB::Install::Configuration::ValueChangeListeners::KeepLSSUserGroupSkipListener;

use strict;
use warnings;

use SDB::Common::Utils qw(getSidcryptName);

sub new {
    my ($class) = @_;
    return bless({}, $class);
}

sub onValueChange {
    my ($self, $keepLSSUserValue, $config) = @_;
    my $lssUserGroupDefaultName = getSidcryptName($config->getSID());
    $self->_setConfig($config);
    $self->_setLssUserGroupDefaultName($lssUserGroupDefaultName);
    $self->_setShouldKeepLssUser($keepLSSUserValue);
    $config->setSkip('KeepLSSUserGroup', 1, $self->_getSkipKeepLssUserGroupParamMessage()) if $self->_shouldSkipKeepLssUserGroup();
    return 1;
}

sub _shouldSkipKeepLssUserGroup {
    my ($self) = @_;
    if($self->_shouldKeepLssUser() || !$self->_isLssUserGroupNameTheDefaultOne()) {
        return 1;
    }
    return 0;
}

sub _getSkipKeepLssUserGroupParamMessage {
    my ($self) = @_;
    my $lssUserGroupDefaultName = $self->_getLssUserGroupDefaultName();

    if(!$self->_isLssUserGroupNameTheDefaultOne()) {
        return "It is only applicable when the group name is '$lssUserGroupDefaultName'.";
    }
    return undef;
}

sub _determineIfLssUserGroupNameIsTheDefaultOne {
    my ($self) = @_;
    my $config = $self->_getConfig();
    my $lssUserGroupName = $config->getLssUserGroupName();
    return 0 if !defined $lssUserGroupName;

    return $lssUserGroupName eq $self->_getLssUserGroupDefaultName();
}

sub _isLssUserGroupNameTheDefaultOne {
    my ($self) = @_;
    if(!defined $self->{isLssUserGroupNameTheDefaultOne}) {
        $self->{isLssUserGroupNameTheDefaultOne} = $self->_determineIfLssUserGroupNameIsTheDefaultOne();
    }
    return $self->{isLssUserGroupNameTheDefaultOne};
}

sub _getLssUserGroupDefaultName {
    my ($self) = @_;
    return $self->{lssUserGroupDefaultName};
}

sub _setLssUserGroupDefaultName {
    my ($self, $groupDefaultName) = @_;
    $self->{lssUserGroupDefaultName} = $groupDefaultName;
    return 1;
}

sub _shouldKeepLssUser {
    my ($self) = @_;
    return $self->{shouldKeepLssUser};
}

sub _setShouldKeepLssUser {
    my ($self, $shouldKeepLssUser) = @_;
    $self->{shouldKeepLssUser} = $shouldKeepLssUser;
    return 1;
}

sub _getConfig {
    my ($self) = @_;
    return $self->{config};
}

sub _setConfig {
    my ($self, $config) = @_;
    $self->{config} = $config;
    return 1;
}

1;