package SDB::Install::Configuration::ValueChangeListeners::LSSUserIDListener;

use strict;
use warnings;
use SDB::Common::BuiltIn;
use SDB::Install::Version;
use SDB::Install::Configuration::ExternalProgramConfiguration;

use File::Spec;


use constant MIN_SDK_VERSION => '2.6.36';

sub new {
    my ($class) = @_;
    return bless({}, $class);
}

sub onValueChange {
    my ($self, $value, $config) = @_;
    return 1 if (!$self->_checkLcmSdkVersion($config));

    my $userName = SDB::Common::BuiltIn->get_instance()->getpwuid($value);
    if(!$userName) {
        $config->setSkip('LSSPassword', 0);
        return 1;
    }

    my @hostsWithoutLSSUser = $config->getRemoteHostsWithoutUser($userName);
    if (@hostsWithoutLSSUser) {
        my $hostNames = join(',', @hostsWithoutLSSUser);
        $config->getMsgLst()->addMessage("Paramater 'LSSPassword' is unskipped, because user '$userName' does not exist on the following host(s): $hostNames");
        $config->setSkip('LSSPassword', 0);
    } else {
        $config->setSkip('LSSPassword', 1);
    }
    return 1;
}

sub _checkLcmSdkVersion {
    my ($self, $config) = @_;
    my $hdbInstance = $config->getOwnInstance();
    my $lssInstance = defined $hdbInstance ? $hdbInstance->getLssInstance() : undef;
    return 0 if(!defined $lssInstance);

    my $lssExecutable = File::Spec->catfile($lssInstance->getLssSidDir(), 'install', 'hdbinst');
    my $externalConfig = SDB::Install::Configuration::ExternalProgramConfiguration->new($lssExecutable);
    $externalConfig->setMsgLstContext($config->getMsgLstContext());
    my $rc = $externalConfig->init();
    if (!defined $rc){
        $config->getMsgLst()->addWarning("Cannot detect LCMsdk version.", $externalConfig->getErrMsgLst());
        return 0;
    }
    my $version = $externalConfig->getInstallerVersion();
    my $minVersion = SDB::Install::Version->new(split('\.', MIN_SDK_VERSION));
    my $sdkVersion = SDB::Install::Version->new(split('\.', $version));
    return !$sdkVersion->isOlderThan($minVersion);
}

1;
