package SDB::Install::IniFileBase;

use strict;
use base 'SDB::Install::BaseLegacy';
use SDB::Install::Globals qw ($gBoolTruePattern $gBoolFalsePattern);
use SDB::Install::Tools qw (ltrim rtrim getFirstLowercaseMatch);
use SDB::Common::Utils qw(addAssertionError);

sub getSections {
    my ($self) = @_;
    if (defined $self->{data}){
        return [keys %{$self->{data}}];
    }
    return [];
}

sub getBoolValue {
    my ($self, $section, $key, $rlayer) = @_;
    my $value = $self->getValue($section, $key, $rlayer, 1);
    return undef if (!defined $value);
    return $value =~ /$gBoolTruePattern/i;
}

sub isBoolValue {
    my ($self, $value) = @_;
    my $isTrueValue = $value =~ /$gBoolTruePattern/i;
    my $isFalseValue = $value =~ /$gBoolFalsePattern/i;
    return $isTrueValue || $isFalseValue;
}

sub _parseKeyValuePair {
    my ($self, $line, $section) = @_;
    my ($key, $value) = ($line =~ /^([^=]+)=(.*)/);
    rtrim (\$key);
    ltrim (\$value);

    if ($key) {
        $key = $self->_getRealKey($section, $key);
        return ($key, $value);
    }
    return ();
}

sub addGetBoolValueDebugMessage {
    my ($self, $key, $section, $filename) = @_;
    addAssertionError("getValue used instead of getBoolValue for the boolean parameter '$key' (section '$section') in $filename", $self);
    return 1;
}

sub _getRealSection {
    my ($self, $section) = @_;
    return getFirstLowercaseMatch($self->getSections(), $section) // $section;
}

sub write {
    ...
}

sub setValue {
    ...
}

sub getValue {
    ...
}

sub removeKey {
    ...
}

sub _getRealKey {
    ...
}

1;