package SDB::Install::LSS::LssSidcryptUser;

use strict;
use warnings;

use SDB::Install::System qw(makedir copy_file changeOwn isAdmin);
use parent qw(SDB::Install::User);
use File::stat qw(stat);
use File::Spec;
use SDB::Install::Globals qw($gProductName);
use experimental qw(smartmatch);

sub new{
    return shift->SUPER::new(@_);
}

sub configureHome{
    my ($self, $sid, $installationPath) = @_;

    my $home = $self->home();

    my %copyitems = (
        'CSHRC' => '.cshrc',
        'PROFILE' => '.profile',
        'BASHRC' => '.bashrc',
        'SAPENV.SH' => '.sapenv.sh',
        'SAPENV.CSH' => '.sapenv.csh'
    );

    my $msg = $self->getMsgLst()->addMessage ("Configuring user home '$home'");
    $msg->setEndTag ('Configure Home Dir');
    my $submsglst = $msg->getSubMsgLst ();

    if (!File::stat::stat($home)){
        my $config = {'uid' => $self->id(), 'gid' => $self->gid(), 'mode' => 0750};
        if (!defined SDB::Install::System::makedir ($home,$config)){
            $self->setErrorMessage ("Cannot create home directory $home" , $config);
            $msg->endMessage ();
            return undef;
        }

    }
    my $dst;
    my $srcScriptsDir = File::Spec->catdir($installationPath,'exe','env');
    foreach my $src (keys %copyitems){
        $dst = File::Spec->catfile($home,$copyitems{$src});
        $src = File::Spec->catfile($srcScriptsDir , $src);
        if (File::stat::stat($dst)){
            $submsglst->addMessage ("$dst already exists -> overwriting");
        }

        $submsglst->addMessage ("Copy $src to $dst");

        my $config = {};
        if (!SDB::Install::System::copy_file ($src,$dst, $config)){
            $submsglst->addWarning ("Could not copy $dst", $config);
        }
    }

    my $errlst = new SDB::Install::MsgLst ();
    if (!SDB::Install::System::changeOwn (undef, $self->id(), undef, $self->gid(), $self->home(), $errlst, $msg->getSubMsgLst ())){
        $self->setErrorMessage ("Change owner of home directory '$home' failed", $errlst);
        return undef;
    }
    $msg->endMessage ();
    return 1;
}

# Override
sub verifyPassword {
    my ($self, $password, $instconfig) = @_;
    my $str = $instconfig->getString('LSSPassword');
    if (!$password || length($password) < 8) {
        $instconfig->appendErrorMessage("$str must contain at least 8 characters");
        return 0;
    }
    return 1;
}

1;