package SDB::Install::PythonExecutor;

use strict;
use warnings;

use base qw (SDB::Install::BaseLegacy);
use SDB::Install::System;
use SDB::Install::SysVars;
use SDB::Install::System qw(getPythonEnvironmentVariablesHash);

our $python;

sub findSystemPython {
    if (!$python){
        $python = which ('python');
        if (!defined $python && !$isWin){
            local %ENV = %ENV;
            $ENV{PATH} = join ($env_path_separator, qw(/usr/bin /usr/local/bin));
            $python = which ('python');
        }
    }
    return $python;
}

sub new{
    my $self = shift->SUPER::new ();
    $self->setPython(findSystemPython());
    $self->setStdOutLines();
    return $self;
}

sub setScript{
    if (-f $_[1]){
        $_[0]->{'script'} = $_[1];
        return 1;
    }
    $_[0]->AddError ("'$_[1]' $!");
    return undef;
}

sub getScript {
    my ($self) = @_;
    return $self->{script};
}


sub runScript{
    my ($self, $args, $uid, $gid) = @_;
    $self->setStdOutLines();
    my $cfg = {};
    if (defined $uid) {
        $cfg->{UID} = $uid;
    }
    if (defined $gid) {
        $cfg->{GID} = $gid;
    }
    $cfg->{outLines} = [];
    if (!defined $args){
        $args = [];
    }

    local %ENV = %ENV;
    $self->setEnvironment(\%ENV);

    if (!defined $self->getPython()){
        $self->AddError ("No python interpreter defined\n");
        return undef;
    }
    if (!defined $self->getScript()){
        $self->AddError ("No python script defined\n");
        return undef;
    }
    my $rc = exec_program ($self->getPython(), [$self->getScript(), @$args], $cfg);
    if (!defined $rc || $rc != 0){
        $self->getErrMsgLst->addError ("Performing python script failed", $cfg);
        $self->AddMessage (undef, $cfg);
        return undef;
    }
    $self->AddMessage (undef, $cfg);
    $self->setStdOutLines($cfg->{outLines});
    return 1;
}

sub setStdOutLines {
    my ($self, $stdOutLines) = @_;
    $self->{stdOutLines} = $stdOutLines;
}

sub getStdOutLines {
    return $_[0]->{stdOutLines};
}

sub setPython {
    my ($self, $pythonPath) = @_;
    $self->{python} = $pythonPath;
}

sub getPython {
    my ($self) = @_;
    return $self->{python};
}

sub setPythonExeDir {
    my ($self, $pythonExeDir) = @_;
    $self->{pythonExeDir} = $pythonExeDir;
}

sub getPythonExeDir {
    my ($self) = @_;
    return $self->{pythonExeDir};
}

sub setPythonVersion {
    my ($self, $pythonVersion) = @_;
    $self->{pythonVersion} = $pythonVersion;
}

sub getPythonVersion {
    my ($self) = @_;
    return $self->{pythonVersion};
}

sub setEnvironment {
    my ($self, $env) = @_;
    return if !($self->getPythonVersion() && $self->getPythonExeDir());
    my $pythonEnvVars = getPythonEnvironmentVariablesHash($self->getPythonVersion(), $self->getPythonExeDir(), $env->{PYTHONPATH});
    $self->setPython($pythonEnvVars->{PYTHON_EXE});
    $env->{PYTHONDONTWRITEBYTECODE} = 'x';
    $env->{PYTHONHOME} = $pythonEnvVars->{PYTHONHOME};
    $env->{PYTHONPATH} = $pythonEnvVars->{PYTHONPATH};
    $env->{PATH} = defined ($env->{PATH})
                   ? join($env_path_separator, $pythonEnvVars->{PATH}, $env->{PATH})
                   : $pythonEnvVars->{PATH};
    $env->{LD_LIBRARY_PATH} = defined ($env->{LD_LIBRARY_PATH})
                              ? join($env_path_separator, $env->{LD_LIBRARY_PATH}, $pythonEnvVars->{LD_LIBRARY_PATH})
                              : $pythonEnvVars->{LD_LIBRARY_PATH};
}

1;

