package SDB::Install::Utils::ParameterMigration::GlobalIniMigratableParameters;

use strict;

sub new {
    my ($class) = @_;
    my $self = {};
    bless $self, $class;

    $self->_setMigratableParameters();
    return $self;
}

sub _setMigratableParameters {
    my ($self) = @_;
    $self->{parametersToMigrate} = {
        'EnableSsl' => $self->getParamEnableSsl(),
        'SslMinProtocolVersion' => $self->getParamSslMinProtocolVersion(),
    };
    return 1;
}

sub getParametersList {
    my ($self) = @_;
    my @parametersList = sort keys %{$self->{parametersToMigrate}};
    return \@parametersList;
}

sub getParameter {
    my ($self, $paramKey) = @_;
    my $parametersToMigrateHash = $self->_getMigratableParametersHash();
    return $parametersToMigrateHash->{$paramKey};
}

sub _getMigratableParametersHash {
    my ($self) = @_;
    return $self->{parametersToMigrate};
}

sub getParamEnableSsl {
    return {
        'sections' => ['system_replication_communication'],
        'iniKey'   => 'enable_ssl',
    };
}

sub getParamSslMinProtocolVersion {
    return {
        'sections' => ['communication', 'ldap'],
        'iniKey'   => 'sslMinProtocolVersion',
    };
}

1;